from django.core.files import File
from django.http import HttpResponse, HttpResponseForbidden
import magic

from .. import settings


def get_media_response(request, data, permissions=None, headers=None):
    # For compatibility purpose
    content, url = None, None
    if isinstance(data, (File, )):
        content, url = data, data.url
    else:
        # https://docs.djangoproject.com/fr/2.1/ref/request-response/#passing-iterators # noqa
        content, url = open(data['path'], mode='rb'), data['url']

    filetype = magic.from_buffer(content.read(1024), mime=True)
    content.seek(0)

    if isinstance(permissions, list):
        if not set(permissions).intersection(
                request.user.get_all_permissions()):
            return HttpResponseForbidden()

    response = HttpResponse(content_type='application/octet-stream')
    if isinstance(headers, dict):
        for header, value in headers.items():
            response[header] = value

    if settings.MEDIA_ACCEL_REDIRECT:
        response['X-Accel-Redirect'] = f'{url}'
    else:
        response.content = content.read()
        response.content_type = filetype

    return response



from rest_framework import generics

from terra_settings.filters import JSONFieldOrderingFilter
from terra_settings.tests.test_app.models import DummyModel
from terra_settings.tests.test_app.serializers import DummySerializer


class OrderingListView(generics.ListAPIView):
    permission_classes = ()
    queryset = DummyModel.objects.all().order_by('pk')
    serializer_class = DummySerializer
    filter_backends = (JSONFieldOrderingFilter,)
    ordering_fields = ['properties', ]



from django.urls import reverse
from rest_framework.test import APITestCase


class SettingsViewTestCase(APITestCase):
    def test_view(self):
        response = self.client.get(reverse('settings'))
        self.assertEqual(200, response.status_code)
        self.assertListEqual(
            ['language', 'base_layers', ],
            list(response.json())
        )



from django.conf import settings

from .helpers import Choices

STATES = Choices(
    ('DRAFT', 100, 'Draft'),
    ('SUBMITTED', 200, 'Submitted'),
    ('ACCEPTED', 300, 'Accepted'),
    ('REFUSED', -1, 'Refused'),
    ('CANCELLED', -100, 'Cancelled'),
    ('MISSING', 0, 'Missing'),
)
STATES.add_subset('MANUAL', (
    'DRAFT',
    'SUBMITTED',
    'ACCEPTED',
    'REFUSED',
    'CANCELLED',
))

STATES = getattr(settings, 'STATES', STATES)

TERRA_APPLIANCE_SETTINGS = getattr(settings, 'TERRA_APPLIANCE_SETTINGS', {})
FRONT_URL = getattr(settings, 'FRONT_URL', 'http://localhost:3000')
HOSTNAME = getattr(settings, 'HOSTNAME', 'http://localhost:8000')

MEDIA_ACCEL_REDIRECT = getattr(settings, 'MEDIA_ACCEL_REDIRECT', False)



from rest_framework import serializers

from terra_settings.tests.test_app.models import DummyModel


class DummySerializer(serializers.ModelSerializer):
    class Meta:
        model = DummyModel
        fields = ('name', 'properties')



from django.test import TestCase

from terra_settings.tests.test_app.models import DummyModel
from terra_settings.tests.test_app.serializers import DummySerializer

from rest_framework import generics
from rest_framework.settings import api_settings
from rest_framework.test import APIRequestFactory

from terra_settings.filters import JSONFieldOrderingFilter

factory = APIRequestFactory()


class JSONOrderingTestCase(TestCase):

    def setUp(self):
        for idx in range(3):
            data = {
                'name': 'a' * (idx + 1),
                'properties': {
                    'key': idx,
                }
            }
            # using Layer model as fake tests models needs lot of development
            DummyModel.objects.create(**data)

    def test_json_ordering(self):
        class OrderingListView(generics.ListAPIView):
            permission_classes = ()
            queryset = DummyModel.objects.all().order_by('pk')
            serializer_class = DummySerializer
            filter_backends = (JSONFieldOrderingFilter, )
            ordering_fields = ['properties', ]

        view = OrderingListView.as_view()

        # testing ascending
        request = factory.get('/',
                              {api_settings.ORDERING_PARAM: 'properties__key'})
        response = view(request)

        self.assertListEqual(
            [0, 1, 2],
            [i['properties']['key'] for i in response.data['results']])

        # testing descending
        request = factory.get('/',
                              {api_settings.ORDERING_PARAM: '-properties__key'})
        response = view(request)

        self.assertListEqual(
            [2, 1, 0],
            [i['properties']['key'] for i in response.data['results']])

        request = factory.get('/', {api_settings.ORDERING_PARAM: 'name'})
        response = view(request)

        # testing normal field
        self.assertListEqual(
            [0, 1, 2],
            [i['properties']['key'] for i in response.data['results']])



from django.contrib.postgres.fields import JSONField
from django.db import models


class DummyModel(models.Model):
    name = models.CharField(max_length=250)
    properties = JSONField(default=dict)



from io import StringIO
import os

from django.core.files.uploadedfile import SimpleUploadedFile
from django.test import TestCase
from django.test.client import RequestFactory
from rest_framework import status

from terra_settings.helpers import Choices, CustomCsvBuilder
from terra_settings.helpers.responses import get_media_response


class CSVTestCase(TestCase):
    def test_create_csv_1(self):
        data = {"test": "hello-world",
                "value": [
                    {"data": {
                        "test1": {
                            "first": {
                                "events": 1,
                                "participants": 1},
                            "second": {
                                "events": 2,
                                "participants": 2},
                            "third": {
                                "events": 1,
                                "participants": 101}
                        }
                    },
                        "name": "Name1"
                    },
                    {"data": {
                        "test0": {
                            "fourth": {
                                "events": 1,
                                "participants": 1
                            }
                        },
                        "test1": {
                            "first": {
                                "events": 1,
                                "participants": 1
                            },
                            "second": {
                                "events": 8,
                                "participants": 73
                            },
                            "third": {
                                "events": 1,
                                "participants": 101}
                        }
                    },
                        "name": "Name2"}
                ]
                }

        csv_builder = CustomCsvBuilder(data)

        output = StringIO()
        csv_builder.create_csv(output)
        list_lines = self.get_columns(output.read())
        self.assertCountEqual(
            list_lines,
            [['test', 'hello-world', ''],
             ['value', '', ''],
             ['value_data_test0_fourth_events', '', '1'],
             ['value_data_test0_fourth_participants', '', '1'],
             ['value_data_test1_first_events', '1', '1'],
             ['value_data_test1_first_participants', '1', '1'],
             ['value_data_test1_second_events', '2', '8'],
             ['value_data_test1_second_participants', '2', '73'],
             ['value_data_test1_third_events', '1', '1'],
             ['value_data_test1_third_participants', '101', '101'],
             ['value_name', 'Name1', 'Name2']])

    def test_create_csv_2(self):
        data = {"test": "hello-world", "test2": "HELLO-WORLD"}
        csv_builder = CustomCsvBuilder(data)
        output = StringIO()
        csv_builder.create_csv(output)
        list_lines = self.get_columns(output.read())
        self.assertCountEqual(
            list_lines,
            [['test', 'hello-world'], ['test2', 'HELLO-WORLD']])

    def get_columns(self, csv):
        list_csv = []
        tab = csv.split('\r\n')
        tab.pop(-1)
        for line in tab:
            values = line.split(',')
            list_csv.append(values)
        by_column = []
        for id in range(len(list_csv[0])):
            column = []
            for line in list_csv:
                column.append(line[id])
            by_column.append(column)
        return by_column


class ChoicesTests(TestCase):
    def setUp(self):
        self.MY_CHOICES = Choices(
            ('ONE', 1, 'One for the money'),
            ('TWO', 2, 'Two for the show'),
            ('THREE', 3, 'Three to get ready'),
        )
        self.MY_CHOICES.add_subset("ODD", ("ONE", "THREE"))

    """
    Testing the choices
    """

    def test_simple_choice(self):
        self.assertEqual(self.MY_CHOICES.CHOICES,
                         ((1, "One for the money"),
                          (2, "Two for the show"),
                          (3, "Three to get ready"),))
        self.assertEqual(self.MY_CHOICES.CHOICES_DICT,
                         {
                             1: 'One for the money',
                             2: 'Two for the show',
                             3: 'Three to get ready'
                         })
        self.assertEqual(self.MY_CHOICES.REVERTED_CHOICES_DICT,
                         {
                             'One for the money': 1,
                             'Three to get ready': 3,
                             'Two for the show': 2
                         })

    def test__contains__(self):
        self.failUnless(self.MY_CHOICES.ONE in self.MY_CHOICES)

    def test__iter__(self):
        self.assertEqual([k for k, v in self.MY_CHOICES], [1, 2, 3])

    def test_unique_values(self):
        self.assertRaises(ValueError, Choices,
                          ('TWO', 4, 'Deux'), ('FOUR', 4, 'Quatre'))

    def test_unique_constants(self):
        self.assertRaises(ValueError, Choices,
                          ('TWO', 2, 'Deux'), ('TWO', 4, 'Quatre'))

    def test_const_choice(self):
        self.assertEqual(self.MY_CHOICES.CONST_CHOICES,
                         (("ONE", "One for the money"),
                          ("TWO", "Two for the show"),
                          ("THREE", "Three to get ready"),))

    def test_value_to_const(self):
        self.assertEqual(self.MY_CHOICES.VALUE_TO_CONST,
                         {1: "ONE", 2: "TWO", 3: "THREE"})

    def test_add_should_add_in_correct_order(self):
        SOME_CHOICES = Choices(
            ('ONE', 1, 'One'),
            ('TWO', 2, 'Two'),
        )
        OTHER_CHOICES = Choices(
            ('THREE', 3, 'Three'),
            ('FOUR', 4, 'Four'),
        )
        # Adding a choices to choices
        tup = SOME_CHOICES + OTHER_CHOICES
        self.assertEqual(tup, ((1, 'One'), (2, 'Two'),
                               (3, 'Three'), (4, 'Four')))

        # Adding a tuple to choices
        tup = SOME_CHOICES + ((3, 'Three'), (4, 'Four'))
        self.assertEqual(tup, ((1, 'One'), (2, 'Two'),
                               (3, 'Three'), (4, 'Four')))

        """Adding a choices to tuple => do not work; is it possible to
           emulate it?
            tup = ((1, 'One'), (2, 'Two')) + OTHER_CHOICES
            self.assertEqual(tup, ((1, 'One'), (2, 'Two'),
                                   (3, 'Three'), (4, 'Four')))
        """

    def test_retrocompatibility(self):
        MY_CHOICES = Choices(
            ('TWO', 2, 'Deux'),
            ('FOUR', 4, 'Quatre'),
            name="EVEN"
        )
        MY_CHOICES.add_choices("ODD",
                               ('ONE', 1, 'Un'),
                               ('THREE', 3, 'Trois'), )
        self.assertEqual(MY_CHOICES.CHOICES, ((2, 'Deux'), (4, 'Quatre'),
                                              (1, 'Un'), (3, 'Trois')))
        self.assertEqual(MY_CHOICES.ODD, ((1, 'Un'), (3, 'Trois')))
        self.assertEqual(MY_CHOICES.EVEN, ((2, 'Deux'), (4, 'Quatre')))


class SubsetTests(TestCase):
    def setUp(self):
        self.MY_CHOICES = Choices(
            ('ONE', 1, 'One for the money'),
            ('TWO', 2, 'Two for the show'),
            ('THREE', 3, 'Three to get ready'),
        )
        self.MY_CHOICES.add_subset("ODD", ("ONE", "THREE"))
        self.MY_CHOICES.add_subset("ODD_BIS", ("ONE", "THREE"))

    def test_basic(self):
        self.assertEqual(self.MY_CHOICES.ODD, ((1, 'One for the money'),
                                               (3, 'Three to get ready')))

    def test__contains__(self):
        self.failUnless(self.MY_CHOICES.ONE in self.MY_CHOICES.ODD)

    def test__eq__(self):
        self.assertEqual(self.MY_CHOICES.ODD, ((1, 'One for the money'),
                                               (3, 'Three to get ready')))
        self.assertEqual(self.MY_CHOICES.ODD, self.MY_CHOICES.ODD_BIS)


class MediaResponseTest(TestCase):
    def setUp(self):
        self.factory = RequestFactory()

    def test_get_media_response_with_path(self):
        request = self.factory.get('fake/path')

        # Creating a real file
        # In this case, we test if get_media_response return content
        # from a file and so will use open()
        tmp_name = '/tmp/test.txt'
        with open(tmp_name, 'wb') as tmp_file:
            tmp_file.write(b"ceci n'est pas une pipe")

        response = get_media_response(
            request,
            {'path': tmp_name, 'url': None}  # url none, no accel-redirect
        )

        # deleting the file since we don't need it anymore
        os.remove(tmp_name)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIsInstance(response.content, bytes)
        self.assertEqual(response.content, b"ceci n'est pas une pipe")

    def test_get_media_response_with_file_object(self):
        request = self.factory.get('fake/path')

        tmp_file = SimpleUploadedFile(name='/tmp/file.txt',
                                      content=b'creativity takes courage')

        # Adding fake url, we don't care, we are not testing accel-redirect
        tmp_file.url = None

        response = get_media_response(request, tmp_file)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIsInstance(response.content, bytes)
        self.assertEqual(response.content, b'creativity takes courage')



#!/usr/bin/env python

import os
from setuptools import setup, find_packages

HERE = os.path.abspath(os.path.dirname(__file__))

README = open(os.path.join(HERE, 'README.md')).read()
CHANGES = open(os.path.join(HERE, 'CHANGES.md')).read()

test_require = [
    'factory-boy',
    'flake8',
    'coverage',
]

setup(
    name='django-terra-settings',
    version=open(os.path.join(HERE, 'terra_settings', 'VERSION.md')).read().strip(),
    include_package_data=True,
    author="Makina Corpus",
    author_email="terralego-pypi@makina-corpus.com",
    description='Common settings and utils for terra apps.',
    long_description=README + '\n\n' + CHANGES,
    description_content_type="text/markdown",
    long_description_content_type="text/markdown",
    packages=find_packages(),
    url='https://github.com/Terralego/django-terra-settings.git',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
    ],
    python_requires='>=3.6',
    tests_require=test_require,
    install_requires=[
        'django>=2.2',
        'djangorestframework',
        "django-url-filter>=0.3",
        "python-magic>=0.4",
        "coreapi",
        "coreschema",
        'psycopg2>=2.8',
        'django-mapbox-baselayer>=0.0.6',
    ],
    extras_require={
        'dev': test_require + [
            'django-debug-toolbar'
        ]
    }
)
